@@ -1,17 +1,20 @@
-# Copyrights 2007-2016 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-
-use warnings;
-use strict;
+# This code is part of distribution Log-Report. Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report;
 use vars '$VERSION';
-$VERSION = '1.18';
+$VERSION = '1.26';
 
 use base 'Exporter';
 
+use warnings;
+use strict;
+
 use List::Util         qw/first/;
 use Scalar::Util       qw/blessed/;
 
@@ -19,8 +22,8 @@ use Log::Report::Util;
 my $lrm = 'Log::Report::Message';
 
 ### if you change anything here, you also have to change Log::Report::Minimal
-my @make_msg         = qw/__ __x __n __nx __xn N__ N__n N__w/;
-my @functions        = qw/report dispatcher try textdomain/;
+my @make_msg   = qw/__ __x __n __nx __xn N__ N__n N__w __p __px __np __npx/;
+my @functions  = qw/report dispatcher try textdomain/;
 my @reason_functions = qw/trace assert info notice warning
    mistake error fault alert failure panic/;
 
@@ -32,6 +35,7 @@ sub mistake(@); sub error(@); sub fault(@); sub alert(@); sub failure(@);
 sub panic(@);
 sub __($); sub __x($@); sub __n($$$@); sub __nx($$$@); sub __xn($$$@);
 sub N__($); sub N__n($$); sub N__w(@);
+sub __p($$); sub __px($$@); sub __np($$$$); sub __npx($$$$@);
 
 #
 # Some initiations
@@ -42,7 +46,7 @@ my $default_mode = 0;
 my @nested_tries;
 
 # we can only load these after Log::Report has compiled, because
-# the use this module themselves.
+# they use this module themselves as well.
 
 require Log::Report::Die;
 require Log::Report::Domain;
@@ -51,8 +55,6 @@ require Log::Report::Exception;
 require Log::Report::Dispatcher;
 require Log::Report::Dispatcher::Try;
 
-#eval "require Log::Report::Translator::POT"; panic $@ if $@;
-#, translator => Log::Report::Translator::POT->new(charset => 'utf-8');
 textdomain 'log-report';
 
 my $default_dispatcher = dispatcher PERL => 'default', accept => 'NOTICE-';
@@ -117,6 +119,14 @@ sub report($@)
     elsif($message->isa('Log::Report::Message'))
     {   @_==0 or error __x"a message object is reported with more parameters";
     }
+    else
+    {   # foreign object
+        my $text = "$message";  # hope stringification is overloaded
+		$text    =~ s/\s*$//gs;
+        @_%2 and error __x"odd length parameter list with object '{msg}'",
+             msg => $text;
+        $message = $lrm->new(_prepend => $text, @_);
+    }
 
     if(my $to = $message->to)
     {   @disp = grep $_->name eq $to, @disp;
@@ -171,7 +181,9 @@ sub dispatcher($@)
         if(!$reopen_disp)
         {   my $has = first {$_->name eq $name} @$disps;
             if(defined $has && $has ne $default_dispatcher)
-            {   trace "not reopening $name";
+            {   my $default = $name eq 'default'
+                   ? ' (refreshing configuration instead)' : '';
+                trace "not reopening $name$default";
                 return $has;
             }
         }
@@ -276,6 +288,9 @@ sub try(&@)
       and report {location => [caller 0]}, PANIC =>
           __x"odd length parameter list for try(): forgot the terminating ';'?";
 
+	unshift @_, mode => 'DEBUG'
+        if $reporter->{needs}{TRACE};
+
     my $disp = Log::Report::Dispatcher::Try->new(TRY => 'try', @_);
     push @nested_tries, $disp;
 
@@ -291,20 +306,24 @@ sub try(&@)
     pop @nested_tries;
 
     my $is_exception = blessed $err && $err->isa('Log::Report::Exception');
-    if($err && !$is_exception && !$disp->wasFatal)
-    {   ($err, my($opts, $reason, $text))
-           = Log::Report::Die::die_decode($err, on_die => $disp->die2reason);
+    if(!$is_exception && $err && !$disp->wasFatal)
+    {   # Decode exceptions which do not origin from Log::Report reports
+        ($err, my($opts, $reason, $text)) = blessed $err
+           ? Log::Report::Die::exception_decode($err)
+           : Log::Report::Die::die_decode($err, on_die => $disp->die2reason);
+
         $disp->log($opts, $reason, __$text);
     }
 
     $disp->died($err)
-        if $err && ($is_exception ? $err->isFatal : 1);
+        if $is_exception ? $err->isFatal : $err;
 
     $@ = $disp;
 
     wantarray ? @ret : $ret;
 }
 
+#------------
 
 sub trace(@)   {report TRACE   => @_}
 sub assert(@)  {report ASSERT  => @_}
@@ -318,39 +337,45 @@ sub alert(@)   {report ALERT   => @_}
 sub failure(@) {report FAILURE => @_}
 sub panic(@)   {report PANIC   => @_}
 
+#-------------
 
-sub _default_domain(@) { pkg2domain $_[0] }
 
 sub __($)
-{   $lrm->new
+{   my ($cpkg, $fn, $linenr) = caller;
+    $lrm->new
       ( _msgid  => shift
-      , _domain => _default_domain(caller)
+      , _domain => pkg2domain($cpkg)
+      , _use    => "$fn line $linenr"
       );
 } 
 
 
 # label "msgid" added before first argument
 sub __x($@)
-{   @_%2 or error __x"even length parameter list for __x at {where}",
-        where => join(' line ', (caller)[1,2]);
+{   my ($cpkg, $fn, $linenr) = caller;
+    @_%2 or error __x"even length parameter list for __x at {where}",
+        where => "$fn line $linenr";
 
     my $msgid = shift;
     $lrm->new
-     ( _msgid  => $msgid
-     , _expand => 1
-     , _domain => _default_domain(caller)
-     , @_
-     );
+      ( _msgid  => $msgid
+      , _expand => 1
+      , _domain => pkg2domain($cpkg)
+      , _use    => "$fn line $linenr"
+      , @_
+      );
 } 
 
 
 sub __n($$$@)
 {   my ($single, $plural, $count) = (shift, shift, shift);
+    my ($cpkg, $fn, $linenr) = caller;
     $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
-     , _domain => _default_domain(caller)
+     , _domain => pkg2domain($cpkg)
+     , _use    => "$fn line $linenr"
      , @_
      );
 }
@@ -358,12 +383,14 @@ sub __n($$$@)
 
 sub __nx($$$@)
 {   my ($single, $plural, $count) = (shift, shift, shift);
+    my ($cpkg, $fn, $linenr) = caller;
     $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
      , _expand => 1
-     , _domain => _default_domain(caller)
+     , _domain => pkg2domain($cpkg)
+     , _use    => "$fn line $linenr"
      , @_
      );
 }
@@ -371,12 +398,14 @@ sub __nx($$$@)
 
 sub __xn($$$@)   # repeated for prototype
 {   my ($single, $plural, $count) = (shift, shift, shift);
+    my ($cpkg, $fn, $linenr) = caller;
     $lrm->new
      ( _msgid  => $single
      , _plural => $plural
      , _count  => $count
      , _expand => 1
-     , _domain => _default_domain(caller)
+     , _domain => pkg2domain($cpkg)
+     , _use    => "$fn line $linenr"
      , @_
      );
 }
@@ -391,6 +420,26 @@ sub N__n($$) {@_}
 sub N__w(@) {split " ", $_[0]}
 
 
+#-------------
+
+sub __p($$) { __($_[0])->_msgctxt($_[1]) }
+sub __px($$@)
+{   my ($ctxt, $msgid) = (shift, shift);
+    __x($msgid, @_)->_msgctxt($ctxt);
+}
+
+sub __np($$$$)
+{   my ($ctxt, $msgid, $plural, $count) = @_;
+    __n($msgid, $msgid, $plural, $count)->_msgctxt($ctxt);
+}
+
+sub __npx($$$$@)
+{   my ($ctxt, $msgid, $plural, $count) = splice @_, 0, 4;
+    __nx($msgid, $msgid, $plural, $count, @_)->_msgctxt($ctxt);
+}
+
+#-------------
+
 sub import(@)
 {   my $class = shift;
 
@@ -485,11 +534,19 @@ sub translator($;$$$$)
 
 
 sub textdomain(@)
-{   # used for testing
-    return delete $reporter->{textdomains}{$_[0]}
-        if @_==2 && $_[1] eq 'DELETE';
+{   if(@_==1 && blessed $_[0])
+    {   my $domain = shift;
+        $domain->isa('Log::Report::Domain') or panic;
+        return $reporter->{textdomains}{$domain->name} = $domain;
+    }
 
-    my $name   = (@_%2 ? shift : _default_domain(caller)) || 'default';
+    if(@_==2)
+    {    # used for 'maintenance' and testing
+    	return delete $reporter->{textdomains}{$_[0]} if $_[1] eq 'DELETE';
+    	return $reporter->{textdomains}{$_[0]} if $_[1] eq 'EXISTS';
+    }
+
+    my $name   = (@_%2 ? shift : pkg2domain((caller)[0])) || 'default';
     my $domain = $reporter->{textdomains}{$name}
         ||= Log::Report::Domain->new(name => $name);
 
@@ -497,6 +554,7 @@ sub textdomain(@)
     $domain;
 }
 
+#--------------
 
 sub needs(@)
 {   my $thing = shift;
