@@ -1,16 +1,20 @@
-# Copyrights 2007-2016 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report. Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Dispatcher::Try;
 use vars '$VERSION';
-$VERSION = '1.18';
+$VERSION = '1.26';
 
 use base 'Log::Report::Dispatcher';
 
+use warnings;
+use strict;
+
 use Log::Report 'log-report', syntax => 'SHORT';
 use Log::Report::Exception ();
 use Log::Report::Util      qw/%reason_code/;
@@ -33,13 +37,6 @@ sub init($)
     $self;
 }
 
-
-sub close()
-{   my $self = shift;
-    $self->SUPER::close or return;
-    $self;
-}
-
 #-----------------
 
 sub died(;$)
@@ -94,8 +91,8 @@ sub log($$$$)
 
     push @{$self->{exceptions}}, $e;
 
-    $self->{died} ||=
-        exists $opts->{is_fatal} ? $opts->{is_fatal} : $e->isFatal;
+#    $self->{died} ||=
+#        exists $opts->{is_fatal} ? $opts->{is_fatal} : $e->isFatal;
 
     $self;
 }
@@ -106,13 +103,15 @@ sub reportAll(@)   { $_->throw(@_) for shift->exceptions }
 
 #-----------------
 
-sub failed()  {   shift->{died}}
-sub success() { ! shift->{died}}
+sub failed()  {   defined shift->{died}}
+sub success() { ! defined shift->{died}}
+
 
 
 sub wasFatal(@)
 {   my ($self, %args) = @_;
-    $self->{died} or return ();
+    defined $self->{died} or return ();
+
     my $ex = $self->{exceptions}[-1];
     (!$args{class} || $ex->inClass($args{class})) ? $ex : ();
 }
