@@ -1,15 +1,19 @@
-# Copyrights 2007-2016 by [Mark Overmeer].
+# Copyrights 2007-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
-use warnings;
-use strict;
+# This code is part of distribution Log-Report. Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Log::Report::Exception;
 use vars '$VERSION';
-$VERSION = '1.18';
+$VERSION = '1.26';
 
 
+use warnings;
+use strict;
+
 use Log::Report      'log-report';
 use Log::Report::Util qw/is_fatal to_html/;
 use POSIX             qw/locale_h/;
@@ -18,6 +22,7 @@ use Scalar::Util      qw/blessed/;
 
 use overload
     '""'     => 'toString'
+  , 'bool'   => sub {1}    # avoid accidental serialization of message
   , fallback => 1;
 
 
@@ -27,6 +32,7 @@ sub new($@)
     bless \%args, $class;
 }
 
+#----------------
 
 sub report_opts() {shift->{report_opts}}
 
@@ -43,12 +49,14 @@ sub isFatal() { is_fatal shift->{reason} }
 sub message(;$)
 {   my $self = shift;
     @_ or return $self->{message};
-    my $msg = shift;
+
+    my $msg  = shift;
     blessed $msg && $msg->isa('Log::Report::Message')
         or panic "message() of exception expects Log::Report::Message";
     $self->{message} = $msg;
 }
 
+#----------------
 
 sub inClass($) { $_[0]->message->inClass($_[1]) }
 
